import requests
import json
from deso.Route import getRoute

class Diamonds:
    def getDiamonds(publicKey, received = True):
        '''when received is false, it returns the diamonds info which is given by the user.'''
        payload = {"PublicKeyBase58Check":publicKey,
                    "FetchYouDiamonded":not received}
        route = getRoute()
        endpointURL = route + "get-diamonds-for-public-key"
        response = requests.post(endpointURL, json = payload)
        return response.json()
    


import requests
import json
from deso.Route import getRoute

class Posts:

    def  getUserPosts(username = "", publicKey = "",  numToFetch = 10, mediaRequired = False, lastPostHash= "",readerPublicKey = "BC1YLianxEsskKYNyL959k6b6UPYtRXfZs4MF3GkbWofdoFQzZCkJRB"):
        payload = {"PublicKeyBase58Check":publicKey,
                    "Username":username,
                    "ReaderPublicKeyBase58Check":readerPublicKey,
                    "LastPostHashHex":lastPostHash,
                    "NumToFetch":numToFetch,
                    "MediaRequired":mediaRequired}
        ROUTE = getRoute()
        endpointURL = ROUTE + "get-posts-for-public-key"
        response = requests.post(endpointURL, json = payload)
        return response.json()

    def getPostInfo(postHash, commentLimit = 20, fetchParents = False, commentOffset = 0, addGlobalFeedBool = False, readerPublicKey = "BC1YLianxEsskKYNyL959k6b6UPYtRXfZs4MF3GkbWofdoFQzZCkJRB"):
        payload = {"PostHashHex":postHash,
                "ReaderPublicKeyBase58Check":readerPublicKey,
                "FetchParents":fetchParents,
                "CommentOffset":commentOffset,
                "CommentLimit":commentLimit,
                "AddGlobalFeedBool":addGlobalFeedBool}
        ROUTE = getRoute()
        endpointURL = ROUTE + "get-single-post"
        response = requests.post(endpointURL, json = payload)
        return response.json()
            
    def getHiddenPosts(publicKey):
        '''to get all the deleted posts of a user
        API credits: @Kuririn (https://bitclout.com/u/kuririn)'''
        paylod = {"userParams":
                    {"queryParams":
                        {"length":0},
                        "headersParams":{"length":0},
                        "cookiesParams":{"length":0},
                        "bodyParams":{"0":publicKey,"length":1}
                    },    
                    "password":"",
                    "environment":"production",
                    "queryType":"RESTQuery",
                    "frontendVersion":"1",
                    "releaseVersion":None,"includeQueryExecutionMetadata":True}
        response = requests.post("https://apps.tryretool.com/api/public/8952bb20-817f-46f0-b28f-67569f4db682/query?queryName=getHiddenPosts", json= paylod)
        return response.json()



import requests
import json
from deso.Route import getRoute
from deso.Sign import Sign_Transaction


class Trade:
    def __init__(self, seedHex, publicKey):
        self.SEED_HEX = seedHex
        self.PUBLIC_KEY = publicKey

    def buy(self, keyToBuy, DeSo):
        print(self.SEED_HEX)
        DeSoNanos = int(DeSo * (10 ** 9))
        payload = {
            "UpdaterPublicKeyBase58Check": self.PUBLIC_KEY,
            "CreatorPublicKeyBase58Check": keyToBuy,
            "OperationType": "buy",
            "BitCloutToSellNanos": DeSoNanos,
            "CreatorCoinToSellNanos": 0,
            "BitCloutToAddNanos": 0,
            "MinBitCloutExpectedNanos": 0,
            "MinCreatorCoinExpectedNanos": 10,
            "MinFeeRateNanosPerKB": 1000,
        }
        ROUTE = getRoute()
        endpointURL = ROUTE + "buy-or-sell-creator-coin"
        res = requests.post(endpointURL, json=payload)
        transactionHex = res.json()["TransactionHex"]

        signedTransactionHex = Sign_Transaction(
            self.SEED_HEX, transactionHex
        )  # txn signature

        submitPayload = {"TransactionHex": signedTransactionHex}
        endpointURL = ROUTE + "submit-transaction"
        submitResponse = requests.post(endpointURL, json=submitPayload)
        return submitResponse.status_code  # returns 200 if buy is succesful

        # if someone has time pls, add some meaningul try catch blocks and error messages. i will def tip you

    def getMaxCoins(self, publicKeyOfCoin):
        ROUTE = getRoute()
        endpoint = ROUTE + "get-users-stateless"
        payload = {"PublicKeysBase58Check": [self.PUBLIC_KEY]}
        response = requests.post(endpoint, json=payload)
        hodlings = response.json()["UserList"][0]["UsersYouHODL"]
        for hodling in hodlings:
            if hodling["CreatorPublicKeyBase58Check"] == publicKeyOfCoin:
                coinsHeld = hodling["BalanceNanos"]
                if coinsHeld != 0:
                    return hodling["BalanceNanos"]
                else:
                    return -1
        return -1

    def sell(self, keyToSell, coinsToSellNanos=0, sellMax=False):
        coinsToSell = coinsToSellNanos
        if sellMax == True:
            maxCoins = Trade.getMaxCoins(self, publicKeyOfCoin=keyToSell)
            if maxCoins == -1:
                print("You don't hodl that creator")
                return 404
            else:
                coinsToSell = maxCoins

        ROUTE = getRoute()
        payload = {
            "UpdaterPublicKeyBase58Check": self.PUBLIC_KEY,
            "CreatorPublicKeyBase58Check": keyToSell,
            "OperationType": "sell",
            "BitCloutToSellNanos": 0,
            "CreatorCoinToSellNanos": coinsToSell,
            "BitCloutToAddNanos": 0,
            "MinBitCloutExpectedNanos": 0,
            "MinCreatorCoinExpectedNanos": 0,
            "MinFeeRateNanosPerKB": 1000,
        }
        endpointURL = ROUTE + "buy-or-sell-creator-coin"
        res = requests.post(
            endpointURL, json=payload
        )
        transactionHex = res.json()["TransactionHex"]
        signedTransactionHex = Sign_Transaction(
            self.SEED_HEX, transactionHex
        )  # txn signature

        submitPayload = {"TransactionHex": signedTransactionHex}
        endpointURL = ROUTE + "submit-transaction"
        submitResponse = requests.post(endpointURL, json=submitPayload)
        return submitResponse.status_code  # returns 200 if sell is succesful



import requests
from deso.Route import getRoute
from deso.Sign import Sign_Transaction


class Message:
    def __init__(self, seedHex, publicKey):
        self.SEED_HEX = seedHex
        self.PUBLIC_KEY = publicKey

    def send(self, recipient, messageText):
        header = {
            "content-type": "application/json"
        }

        payload = {"SenderPublicKeyBase58Check":self.PUBLIC_KEY,"RecipientPublicKeyBase58Check":recipient,"MessageText":messageText,"MinFeeRateNanosPerKB":1000}
        ROUTE = getRoute()
        endpointURL = ROUTE + "send-message-stateless"
        res = requests.post(endpointURL, json=payload, headers=header)
        transactionHex = res.json()["TransactionHex"]

        signedTransactionHex = Sign_Transaction(
            self.SEED_HEX, transactionHex
        )  # txn signature

        submitPayload = {"TransactionHex": signedTransactionHex}
        endpointURL = ROUTE + "submit-transaction"
        submitResponse = requests.post(endpointURL, json=submitPayload)
        return submitResponse.status_code  # returns 200 if buy is succesful


from deso.Deso import Deso
from deso.Posts import Posts
from deso.Users import Users
from deso.Diamonds import Diamonds
from deso.Trade import Trade
from deso.Post import Post
from deso.Message import Message
from deso.Nft import Nft


import requests
import json
from deso.Route import getRoute
from arweave.arweave_lib import Wallet, Transaction
from arweave.transaction_uploader import get_uploader
import arweave
import logging
import pathlib
from deso.Sign import Sign_Transaction


class Nft:
    def __init__(self, seedHex, publicKey):
        self.SEED_HEX = seedHex
        self.PUBLIC_KEY = publicKey

    def getNFT(postHashHex):
        payload = {"ReaderPublicKeyBase58Check": "BC1YLiwBkXtiG4cf4k4o1VdZHEWT4Caew7HrQ9cKAba5ng5Nev1md1z",
                   "PostHashHex": postHashHex}
        ROUTE = getRoute()
        endpointURL = ROUTE + "get-nft-entries-for-nft-post"
        response = requests.post(endpointURL, json=payload)
        return response.json()

    def uploadToArweave(wallet, image):
        wallet = arweave.Wallet(wallet)
        with open(image, "rb", buffering=0) as file_handler:
            tx = Transaction(
                wallet,  file_handler=file_handler, file_path=image)
            file_extension = pathlib.Path(image).suffix
            type = str(file_extension[1:])
            tx.add_tag('Content-Type', 'image/' + type)
            tx.sign()
            uploader = get_uploader(tx, file_handler)
            while not uploader.is_complete:
                uploader.upload_chunk()
            tx.send()
            image_id = str(tx.id)
            transaction_id = wallet.get_last_transaction_id()
            build_url = 'https://' + \
                transaction_id[1:] + '.arweave.net/' + image_id
            return build_url

  


import requests
import json
from deso.Route import getRoute

class Users:
    def getUserStateless(publicKeyList, skipForLeaderboard = True):
        '''when skipForLeaderboard is true, it doesn't return info like UsersYouHODL,
            followers public key and blocked users'''
        payload = {"PublicKeysBase58Check": publicKeyList,"SkipForLeaderboard":skipForLeaderboard}
        route = getRoute()
        endpointURL = route + "get-users-stateless"
        response = requests.post(endpointURL, json = payload)
        return response.json()

    def getSingleProfile(publicKey= "", username = "",):
        payload = {"PublicKeyBase58Check": publicKey, "Username": username}
        route = getRoute()
        endpointURL = route + "get-single-profile"
        response = requests.post(endpointURL, json = payload)
        return response.json()

    def getProfilePic(publicKey):
        profilePicURL = f'https://bitclout.com/api/v0/get-single-profile-picture/{publicKey}?fallback=https://bitclout.com/assets/img/default_profile_pic.png'
        return profilePicURL

    def getUsersBlocked( publicKey):
        payload = {"PublicKeysBase58Check": [publicKey],"SkipForLeaderboard":False}
        route = getRoute()
        endpointURL = route + "get-users-stateless"
        response = requests.post(endpointURL, json = payload)
        return response.json()["UserList"][0]["BlockedPubKeys"]

    def getWallet(publicKey, includeCreatorCoin = True): 
        #returns $CLOUTs in wallet and Creators coins on whom the user has invested in
        try:
            payload = {"PublicKeysBase58Check": [publicKey],"SkipForLeaderboard":False}
            route = getRoute()
            endpointURL = route + "get-users-stateless"
            response = requests.post(endpointURL, json = payload)
            responseJson = response.json()
            print(response.json())
            finalResponse = {
            }
            if includeCreatorCoin:    
                coinHeld = responseJson["UserList"][0]["UsersYouHODL"]
                finalResponse["CoinsHeldInfo"] = coinHeld

            CloutInWallet = responseJson["UserList"][0]["BalanceNanos"]
            print(CloutInWallet)
            finalResponse["CloutInWalletNanos"] =  CloutInWallet
            return finalResponse
        except Exception as e:
            return response.status_code

    def getHodlers( username =  "", publicKey= "", lastPublicKey= "", numToFetch = 100, fetchAll = False):
        payload = {"PublicKeyBase58Check":publicKey,
                    "Username":username,
                    "LastPublicKeyBase58Check":lastPublicKey,
                    "NumToFetch":numToFetch,
                    "FetchHodlings":False,
                    "FetchAll":fetchAll}
        route = getRoute()
        endpointURL = route + "get-hodlers-for-public-key"
        response = requests.post(endpointURL, json = payload)
        return response.json()








# This file has code for signing a transaction --> Credit: MiniGlome
import hashlib
import hmac

def get_hmac(key, data):
	return hmac.new(key, data, hashlib.sha256).digest()

def hmac_drbg(entropy, string):
	material = entropy + string
	K = b"\x00" * 32
	V = b"\x01" * 32

	K = get_hmac(K, V + b"\x00" + material)
	V = get_hmac(K, V)
	K = get_hmac(K, V + b"\x01" + material)
	V = get_hmac(K, V)

	temp = b""
	while len(temp) < 32:
		V = get_hmac(K, V)
		temp += V

	return temp[:32]

#######

g=(0x79be667ef9dcbbac55a06295ce870b07029bfcdb2dce28d959f2815b16f81798,
	0x483ada7726a3c4655da4fbfc0e1108a8fd17b448a68554199c47d08ffb10d4b8)
p=0xfffffffffffffffffffffffffffffffffffffffffffffffffffffffefffffc2f
n = 0xfffffffffffffffffffffffffffffffebaaedce6af48a03bbfd25e8cd0364141

def point_add(point1, point2):
	# Returns the result of point1 + point2 according to the group law.
	if point1 is None:
		return point2
	if point2 is None:
		return point1

	x1, y1 = point1
	x2, y2 = point2

	if x1 == x2 and y1 != y2:
		return None

	if x1 == x2:
		m = (3 * x1 * x1) * pow(2 * y1, -1, p)
	else:
		m = (y1 - y2) * pow(x1 - x2, -1, p)

	x3 = m * m - x1 - x2
	y3 = y1 + m * (x3 - x1)
	result = (x3 % p, -y3 % p)

	return result

def scalar_mult(k, point):
	# Returns k * point computed using the double and point_add algorithm.
	result = None
	addend = point

	while k:
		if k & 1:
			# Add.
			result = point_add(result, addend)
		# Double.
		addend = point_add(addend, addend)
		k >>= 1

	return result

#######

def to_DER(r, s): # Signature to DER format
	r = bytes.fromhex(r)
	s = bytes.fromhex(s)
	if r[0] >= 0x80:
		r = bytes.fromhex("00")+r
	if s[0] >= 0x80:
		s = bytes.fromhex("00")+s
	res = bytes.fromhex("02"+hex(len(r))[2:]) + r + bytes.fromhex("02"+hex(len(s))[2:]) + s
	res = bytes.fromhex("30"+hex(len(res))[2:]) + res

	return res.hex()

def hexify(n):
	n = hex(n)[2:]
	if len(n)%2 != 0:
		n = "0"+n
	return n


def Sign_Transaction(seedHex, TransactionHex):
	s256 = hashlib.sha256(hashlib.sha256(bytes.fromhex(TransactionHex)).digest()).digest()
	drbg = hmac_drbg(entropy=bytes.fromhex(seedHex), string=s256)
	k = int.from_bytes(drbg, 'big')
	kp = scalar_mult(k, g)
	kpX = kp[0]
	r = kpX % n
	s = pow(k, -1, n) * (r * int(seedHex, 16)+int(s256.hex(), 16))
	s = s % n
	signature = to_DER(hexify(r), hexify(s))
	signed_transaction = TransactionHex[:-2] + hex(len(bytearray.fromhex(signature)))[2:] + signature
	
	return signed_transaction



import json
import requests

class Deso:
    def getDeSoPrice(): #returns deso price
        response = requests.get("https://api.blockchain.com/v3/exchange/tickers/CLOUT-USD")
        return response.json()
    



import requests
from deso.Route import getRoute
from deso.Sign import Sign_Transaction
import binascii
from ecdsa import SigningKey, SECP256k1
from ecdsa.ecdsa import Public_key
import jwt

class Post:
    def __init__(self, seedHex, publicKey):
        self.SEED_HEX = seedHex
        self.PUBLIC_KEY = publicKey

    def uploadImage(self, fileList):
        ROUTE = getRoute()
        private_key = bytes(self.SEED_HEX, 'utf-8')
        private_key = binascii.unhexlify(private_key)
        key = SigningKey.from_string(private_key, curve=SECP256k1)
        key = key.to_pem()
        encoded_jwt = jwt.encode({}, key, algorithm="ES256")
        # print(encoded_jwt)
        endpointURL = ROUTE + "upload-image"
        payload = {'UserPublicKeyBase58Check': self.PUBLIC_KEY,
                   'JWT': encoded_jwt}
        response = requests.request("POST", endpointURL, data=payload, files=fileList)
        return response.text


    def send(self, content, imageUrl=[]):
        header = {
            "content-type": "application/json"
        }

        payload = {"UpdaterPublicKeyBase58Check": self.PUBLIC_KEY,
                "PostHashHexToModify": "",
                "ParentStakeID": "",
                "Title": "",
                "BodyObj": {"Body": content, "ImageURLs": imageUrl},
                "RecloutedPostHashHex": "",
                "PostExtraData": {},
                "Sub": "",
                "IsHidden":  False,
                "MinFeeRateNanosPerKB": 1000
                }
        ROUTE = getRoute()
        endpointURL = ROUTE + "submit-post"
        res = requests.post(endpointURL, json=payload, headers=header)
        transactionHex = res.json()["TransactionHex"]

        signedTransactionHex = Sign_Transaction(
            self.SEED_HEX, transactionHex
        )  # txn signature

        submitPayload = {"TransactionHex": signedTransactionHex}
        endpointURL = ROUTE + "submit-transaction"
        submitResponse = requests.post(endpointURL, json=submitPayload)
        return {"status":submitResponse.status_code , "postHashHex": submitResponse.json()["TxnHashHex"]} # returns 200

    def mint(self, postHashHex, minBidDeSo,  copy=1, creatorRoyality=5, coinHolderRoyality=10, isForSale=True):
            ROUTE = getRoute()
            endpointURL = ROUTE + "create-nft"
            payload = {"UpdaterPublicKeyBase58Check": self.PUBLIC_KEY,
                        "NFTPostHashHex": postHashHex,
                        "NumCopies": copy,
                        "NFTRoyaltyToCreatorBasisPoints": round(creatorRoyality*100),
                        "NFTRoyaltyToCoinBasisPoints": round(coinHolderRoyality*10), 
                        "HasUnlockable": False, 
                        "IsForSale": isForSale, "MinBidAmountNanos": round( minBidDeSo * 1e9),
                        "MinFeeRateNanosPerKB": 1000}
            response = requests.post(endpointURL, json=payload)
            transactionHex = response.json()["TransactionHex"]
            signedTransactionHex = Sign_Transaction(
                self.SEED_HEX, transactionHex
            )  # txn signature

            submitPayload = {"TransactionHex": signedTransactionHex}
            endpointURL = ROUTE + "submit-transaction"
            submitResponse = requests.post(endpointURL, json=submitPayload)
            return submitResponse.status_code  # returns 200 if buy is succesful
